@@ -15,16 +15,17 @@
 
 #include "../units/fluxdensity.h"
 
-#include "../wsclean/imagefilename.h"
 #include "../wsclean/imagingtable.h"
-#include "../wsclean/primarybeam.h"
 #include "../wsclean/wscleansettings.h"
 
 Deconvolution::Deconvolution(const class WSCleanSettings& settings) :
-	_settings(settings), _autoMaskIsFinished(false),
-	_beamSize(0.0), _pixelScaleX(0.0), _pixelScaleY(0.0)
-{
-}
+	_settings(settings), _parallelDeconvolution(settings),
+	_autoMaskIsFinished(false),
+	_imgWidth(0), // these are not yet set the in settings obj -- load later
+	_imgHeight(0),
+	_beamSize(0.0),
+	_pixelScaleX(0), _pixelScaleY(0)
+{ }
 
 Deconvolution::~Deconvolution()
 {
@@ -38,8 +39,8 @@ void Deconvolution::Perform(const class ImagingTable& groupTable, bool& reachedM
 	
 	_imageAllocator->FreeUnused();
 	ImageSet
-		residualSet(&groupTable, *_imageAllocator, _settings.deconvolutionChannelCount, _settings.squaredJoins, _imgWidth, _imgHeight),
-		modelSet(&groupTable, *_imageAllocator, _settings.deconvolutionChannelCount, _settings.squaredJoins, _imgWidth, _imgHeight);
+		residualSet(&groupTable, *_imageAllocator, _settings.deconvolutionChannelCount, _settings.squaredJoins, _settings.linkedPolarizations, _imgWidth, _imgHeight),
+		modelSet(&groupTable, *_imageAllocator, _settings.deconvolutionChannelCount, _settings.squaredJoins, _settings.linkedPolarizations, _imgWidth, _imgHeight);
 		
 	residualSet.LoadAndAverage(*_residualImages);
 	modelSet.LoadAndAverage(*_modelImages);
@@ -52,7 +53,7 @@ void Deconvolution::Perform(const class ImagingTable& groupTable, bool& reachedM
 	{
 		// When we are in the second phase of automasking, don't use
 		// the RMS background anymore
-		_cleanAlgorithm->SetRMSFactorImage(Image());
+		_parallelDeconvolution.SetRMSFactorImage(Image());
 	}
 	else {
 		if(!_settings.localRMSImage.empty())
@@ -70,7 +71,7 @@ void Deconvolution::Perform(const class ImagingTable& groupTable, bool& reachedM
 				if(value != 0.0)
 					value = stddev / value;
 			}
-			_cleanAlgorithm->SetRMSFactorImage(std::move(rmsImage));
+			_parallelDeconvolution.SetRMSFactorImage(std::move(rmsImage));
 		}
 		else if(_settings.localRMS)
 		{
@@ -93,13 +94,13 @@ void Deconvolution::Perform(const class ImagingTable& groupTable, bool& reachedM
 				if(value != 0.0)
 					value = stddev / value;
 			}
-			_cleanAlgorithm->SetRMSFactorImage(std::move(rmsImage));
+			_parallelDeconvolution.SetRMSFactorImage(std::move(rmsImage));
 		}
 	}
 	if(_settings.autoMask && !_autoMaskIsFinished)
-		_cleanAlgorithm->SetThreshold(std::max(stddev * _settings.autoMaskSigma, _settings.deconvolutionThreshold));
+		_parallelDeconvolution.SetThreshold(std::max(stddev * _settings.autoMaskSigma, _settings.deconvolutionThreshold));
 	else if(_settings.autoDeconvolutionThreshold)
-		_cleanAlgorithm->SetThreshold(std::max(stddev * _settings.autoDeconvolutionThresholdSigma, _settings.deconvolutionThreshold));
+		_parallelDeconvolution.SetThreshold(std::max(stddev * _settings.autoDeconvolutionThresholdSigma, _settings.deconvolutionThreshold));
 	integrated.reset();
 	
 	std::vector<ao::uvector<double>> psfVecs(groupTable.SquaredGroupCount());
@@ -111,13 +112,12 @@ void Deconvolution::Perform(const class ImagingTable& groupTable, bool& reachedM
 	
 	if(_settings.useMultiscale)
 	{
-		MultiScaleAlgorithm& algorithm = static_cast<MultiScaleAlgorithm&>(*_cleanAlgorithm.get());
 		if(_settings.autoMask)
 		{
 			if(_autoMaskIsFinished)
-				algorithm.SetAutoMaskMode(false, true);
+				_parallelDeconvolution.SetAutoMaskMode(false, true);
 			else
-				algorithm.SetAutoMaskMode(true, false);
+				_parallelDeconvolution.SetAutoMaskMode(true, false);
 		}
 	}
 	else {
@@ -135,11 +135,11 @@ void Deconvolution::Perform(const class ImagingTable& groupTable, bool& reachedM
 					}
 				}
 			}
-			_cleanAlgorithm->SetCleanMask(_autoMask.data());
+			_parallelDeconvolution.SetCleanMask(_autoMask.data());
 		}
 	}
 		
-	_cleanAlgorithm->ExecuteMajorIteration(residualSet, modelSet, psfs, _imgWidth, _imgHeight, reachedMajorThreshold);
+	_parallelDeconvolution.ExecuteMajorIteration(residualSet, modelSet, psfs, reachedMajorThreshold);
 	
 	if(!reachedMajorThreshold && _settings.autoMask && !_autoMaskIsFinished)
 	{
@@ -155,26 +155,22 @@ void Deconvolution::Perform(const class ImagingTable& groupTable, bool& reachedM
 	}
 	
 	residualSet.AssignAndStore(*_residualImages);
-	modelSet.InterpolateAndStore(*_modelImages, _cleanAlgorithm->Fitter());
+	modelSet.InterpolateAndStore(*_modelImages, _parallelDeconvolution.FirstAlgorithm().Fitter());
 }
 
-void Deconvolution::FreeDeconvolutionAlgorithms()
-{
-	_cleanAlgorithm.reset();
-}
-
-void Deconvolution::InitializeDeconvolutionAlgorithm(const ImagingTable& groupTable, PolarizationEnum psfPolarization, class ImageBufferAllocator* imageAllocator, size_t imgWidth, size_t imgHeight, double pixelScaleX, double pixelScaleY, double beamSize, size_t threadCount)
+void Deconvolution::InitializeDeconvolutionAlgorithm(const ImagingTable& groupTable, PolarizationEnum psfPolarization, class ImageBufferAllocator* imageAllocator, double beamSize, size_t threadCount)
 {
+	_imgWidth = _settings.trimmedImageWidth;
+	_imgHeight = _settings.trimmedImageHeight;
+	_pixelScaleX = _settings.pixelScaleX;
+	_pixelScaleY = _settings.pixelScaleY;
+	
 	_imageAllocator = imageAllocator;
-	_imgWidth = imgWidth;
-	_imgHeight = imgHeight;
 	_psfPolarization = psfPolarization;
 	_beamSize = beamSize;
-	_pixelScaleX = pixelScaleX;
-	_pixelScaleY = pixelScaleY;
 	_autoMaskIsFinished = false;
 	FreeDeconvolutionAlgorithms();
-	
+	_parallelDeconvolution.SetAllocator(imageAllocator);
 	_summedCount = groupTable.SquaredGroupCount();
 	if(_summedCount == 0)
 		throw std::runtime_error("Nothing to clean");
@@ -196,48 +192,58 @@ void Deconvolution::InitializeDeconvolutionAlgorithm(const ImagingTable& groupTa
 			_polarizations.insert(firstSquaredGroup[p].polarization);
 	}
 	
+	std::unique_ptr<class DeconvolutionAlgorithm> algorithm;
+	
 	if(_settings.useMoreSaneDeconvolution)
 	{
-		_cleanAlgorithm.reset(new MoreSane(_settings.moreSaneLocation, _settings.moreSaneArgs, _settings.moreSaneSigmaLevels, _settings.prefixName, *_imageAllocator));
+		algorithm.reset(
+			new MoreSane(_settings.moreSaneLocation, _settings.moreSaneArgs, _settings.moreSaneSigmaLevels, _settings.prefixName, *_imageAllocator, _parallelDeconvolution.GetFFTWManager()));
 	}
 	else if(_settings.useIUWTDeconvolution)
 	{
-		IUWTDeconvolution* method = new IUWTDeconvolution();
-		_cleanAlgorithm.reset(method);
+		IUWTDeconvolution* method = new IUWTDeconvolution(_parallelDeconvolution.GetFFTWManager());
 		method->SetUseSNRTest(_settings.iuwtSNRTest);
+		algorithm.reset(method);
 	}
 	else if(_settings.useMultiscale)
 	{
-		_cleanAlgorithm.reset(new MultiScaleAlgorithm(*_imageAllocator, beamSize, pixelScaleX, pixelScaleY));
-		MultiScaleAlgorithm *algorithm = static_cast<MultiScaleAlgorithm*>(_cleanAlgorithm.get());
-		algorithm->SetManualScaleList(_settings.multiscaleScaleList);
-		algorithm->SetMultiscaleScaleBias(_settings.multiscaleDeconvolutionScaleBias);
-		algorithm->SetMultiscaleNormalizeResponse(_settings.multiscaleNormalizeResponse);
-		algorithm->SetMultiscaleGain(_settings.multiscaleGain);
-		algorithm->SetShape(_settings.multiscaleShapeFunction);
-		algorithm->SetTrackComponents(_settings.saveSourceList);
-		algorithm->SetConvolutionPadding(_settings.multiscaleConvolutionPadding);
-		algorithm->SetUseFastSubMinorLoop(_settings.multiscaleFastSubMinorLoop);
+		MultiScaleAlgorithm *msAlgorithm = 
+			new MultiScaleAlgorithm(*_imageAllocator, _parallelDeconvolution.GetFFTWManager(), beamSize, _pixelScaleX, _pixelScaleY);
+		msAlgorithm->SetManualScaleList(_settings.multiscaleScaleList);
+		msAlgorithm->SetMultiscaleScaleBias(_settings.multiscaleDeconvolutionScaleBias);
+		msAlgorithm->SetMultiscaleNormalizeResponse(_settings.multiscaleNormalizeResponse);
+		msAlgorithm->SetMultiscaleGain(_settings.multiscaleGain);
+		msAlgorithm->SetShape(_settings.multiscaleShapeFunction);
+		msAlgorithm->SetTrackComponents(_settings.saveSourceList);
+		msAlgorithm->SetConvolutionPadding(_settings.multiscaleConvolutionPadding);
+		msAlgorithm->SetUseFastSubMinorLoop(_settings.multiscaleFastSubMinorLoop);
+		algorithm.reset(msAlgorithm);
 	}
 	else
 	{
-		_cleanAlgorithm.reset(new GenericClean(*_imageAllocator, _settings.useClarkOptimization));
+		algorithm.reset(new GenericClean(*_imageAllocator, _parallelDeconvolution.GetFFTWManager(), _settings.useClarkOptimization));
 	}
 	
-	_cleanAlgorithm->SetMaxNIter(_settings.deconvolutionIterationCount);
-	_cleanAlgorithm->SetThreshold(_settings.deconvolutionThreshold);
-	_cleanAlgorithm->SetGain(_settings.deconvolutionGain);
-	_cleanAlgorithm->SetMGain(_settings.deconvolutionMGain);
-	_cleanAlgorithm->SetCleanBorderRatio(_settings.deconvolutionBorderRatio);
-	_cleanAlgorithm->SetAllowNegativeComponents(_settings.allowNegativeComponents);
-	_cleanAlgorithm->SetStopOnNegativeComponents(_settings.stopOnNegativeComponents);
-	_cleanAlgorithm->SetThreadCount(threadCount);
-	_cleanAlgorithm->SetSpectralFittingMode(_settings.spectralFittingMode, _settings.spectralFittingTerms);
+	algorithm->SetMaxNIter(_settings.deconvolutionIterationCount);
+	algorithm->SetThreshold(_settings.deconvolutionThreshold);
+	algorithm->SetGain(_settings.deconvolutionGain);
+	algorithm->SetMGain(_settings.deconvolutionMGain);
+	algorithm->SetCleanBorderRatio(_settings.deconvolutionBorderRatio);
+	algorithm->SetAllowNegativeComponents(_settings.allowNegativeComponents);
+	algorithm->SetStopOnNegativeComponents(_settings.stopOnNegativeComponents);
+	algorithm->SetThreadCount(threadCount);
+	algorithm->SetSpectralFittingMode(_settings.spectralFittingMode, _settings.spectralFittingTerms);
 	
 	ao::uvector<double> frequencies, weights;
 	calculateDeconvolutionFrequencies(groupTable, frequencies, weights);
-	_cleanAlgorithm->InitializeFrequencies(frequencies, weights);
+	algorithm->InitializeFrequencies(frequencies, weights);
+	_parallelDeconvolution.SetAlgorithm(std::move(algorithm));
 	
+	readMask(groupTable);
+}
+
+void Deconvolution::readMask(const ImagingTable& groupTable)
+{
 	if(!_settings.fitsDeconvolutionMask.empty())
 	{
 		FitsReader maskReader(_settings.fitsDeconvolutionMask, true, true);
@@ -261,7 +267,7 @@ void Deconvolution::InitializeDeconvolutionAlgorithm(const ImagingTable& groupTa
 		_cleanMask.assign(_imgWidth*_imgHeight, false);
 		for(size_t i=0; i!=_imgWidth*_imgHeight; ++i)
 			_cleanMask[i] = (maskData[i]!=0.0);
-		_cleanAlgorithm->SetCleanMask(_cleanMask.data());
+		_parallelDeconvolution.SetCleanMask(_cleanMask.data());
 	} else if(!_settings.casaDeconvolutionMask.empty())
 	{
 		if(_cleanMask.empty())
@@ -273,7 +279,7 @@ void Deconvolution::InitializeDeconvolutionAlgorithm(const ImagingTable& groupTa
 				throw std::runtime_error("Specified CASA mask did not have same dimensions as output image!");
 			maskReader.Read(_cleanMask.data());
 		}
-		_cleanAlgorithm->SetCleanMask(_cleanMask.data());
+		_parallelDeconvolution.SetCleanMask(_cleanMask.data());
 	}
 }
 
@@ -298,108 +304,3 @@ void Deconvolution::calculateDeconvolutionFrequencies(const ImagingTable& groupT
 		frequencies[i] /= counts[i];
 }
 
-void Deconvolution::SaveSourceList(const class ImagingTable& table, long double phaseCentreRA, long double phaseCentreDec) const
-{
-	std::string filename = _settings.prefixName + "-sources.txt";
-	if(_settings.useMultiscale)
-	{
-		MultiScaleAlgorithm& algorithm = static_cast<MultiScaleAlgorithm&>(*_cleanAlgorithm);
-		algorithm.GetComponentList().Write(filename, algorithm, _pixelScaleX, _pixelScaleY, phaseCentreRA, phaseCentreDec);
-	}
-	else {
-		_imageAllocator->FreeUnused();
-		ImageSet modelSet(&table, *_imageAllocator, _settings.deconvolutionChannelCount, _settings.squaredJoins, _imgWidth, _imgHeight);
-		modelSet.LoadAndAverage(*_modelImages);
-		ComponentList componentList(_imgWidth, _imgHeight, modelSet);
-		componentList.WriteSingleScale(filename, *_cleanAlgorithm, _pixelScaleX, _pixelScaleY, phaseCentreRA, phaseCentreDec);
-	}
-}
-
-void Deconvolution::correctChannelForPB(ComponentList& list, const ImagingTableEntry& entry) const
-{
-	Logger::Debug << "Correcting source list of channel " << entry.outputChannelIndex << " for beam\n";
-	ImageFilename filename(entry.outputChannelIndex, entry.outputIntervalIndex);
-	filename.SetPolarization(entry.polarization);
-	PrimaryBeam pb(_settings);
-	PrimaryBeamImageSet beam(_imgWidth, _imgHeight, *_imageAllocator);
-	pb.Load(beam, filename);
-	list.CorrectForBeam(beam, entry.outputChannelIndex);
-}
-
-void Deconvolution::loadAveragePrimaryBeam(PrimaryBeamImageSet& beamImages, size_t imageIndex, const ImagingTable& table) const
-{
-	Logger::Debug << "Averaging beam for deconvolution channel " << imageIndex << "\n";
-	
-	beamImages.SetToZero();
-	
-	ImageBufferAllocator::Ptr scratch;
-	_imageAllocator->Allocate(_imgWidth*_imgHeight, scratch);
-	size_t deconvolutionChannels = _settings.deconvolutionChannelCount;
-	
-	/// TODO : use real weights of images
-	size_t count = 0;
-	PrimaryBeam pb(_settings);
-	for(size_t sqIndex=0; sqIndex!=table.SquaredGroupCount(); ++sqIndex)
-	{
-		size_t curImageIndex = (sqIndex*deconvolutionChannels)/table.SquaredGroupCount();
-		if(curImageIndex == imageIndex)
-		{
-			const ImagingTableEntry e = table.GetSquaredGroup(sqIndex).Front();
-			Logger::Debug << "Adding beam at " << e.CentralFrequency()*1e-6 << " MHz\n";
-			ImageFilename filename(e.outputChannelIndex, e.outputIntervalIndex);
-			
-			PrimaryBeamImageSet scratch(_settings.trimmedImageWidth, _settings.trimmedImageHeight, *_imageAllocator);
-			pb.Load(scratch, filename);
-			beamImages += scratch;
-			
-			count++;
-		}
-	}
-	beamImages *= (1.0 / double(count));
-}
-
-void Deconvolution::SavePBSourceList(const class ImagingTable& table, long double phaseCentreRA, long double phaseCentreDec) const
-{
-	std::unique_ptr<ComponentList> list;
-	if(_settings.useMultiscale)
-	{
-		MultiScaleAlgorithm& algorithm = static_cast<MultiScaleAlgorithm&>(*_cleanAlgorithm);
-		list.reset(new ComponentList(algorithm.GetComponentList()));
-	}
-	else {
-		_imageAllocator->FreeUnused();
-		ImageSet modelSet(&table, *_imageAllocator, _settings.deconvolutionChannelCount, _settings.squaredJoins, _imgWidth, _imgHeight);
-		modelSet.LoadAndAverage(*_modelImages);
-		list.reset(new ComponentList(_imgWidth, _imgHeight, modelSet));
-	}
-	
-	if(_settings.deconvolutionChannelCount == 0 ||
-		_settings.deconvolutionChannelCount == table.SquaredGroupCount())
-	{
-		// No beam averaging is required
-		for(size_t i=0; i!=table.SquaredGroupCount(); ++i)
-		{
-			const ImagingTableEntry entry = table.GetSquaredGroup(i).Front();
-			correctChannelForPB(*list, entry);
-		}
-	}
-	else {
-		for(size_t ch=0; ch!=_settings.deconvolutionChannelCount; ++ch)
-		{
-			PrimaryBeamImageSet beamImages(_imgWidth, _imgHeight, *_imageAllocator);
-			Logger::Debug << "Correcting source list of channel " << ch << " for averaged beam\n";
-			loadAveragePrimaryBeam(beamImages, ch, table);
-			list->CorrectForBeam(beamImages, ch);
-		}
-	}
-	
-	std::string filename = _settings.prefixName + "-sources-pb.txt";
-	if(_settings.useMultiscale)
-	{
-		MultiScaleAlgorithm& algorithm = static_cast<MultiScaleAlgorithm&>(*_cleanAlgorithm);
-		list->Write(filename, algorithm, _pixelScaleX, _pixelScaleY, phaseCentreRA, phaseCentreDec);
-	}
-	else {
-		list->WriteSingleScale(filename, *_cleanAlgorithm, _pixelScaleX, _pixelScaleY, phaseCentreRA, phaseCentreDec);
-	}
-}
