@@ -127,7 +127,7 @@ void PartitionedMS::NextRow()
 		else
 			_metaPtrIsOk = true;
 		
-		if(_weightPtrIsOk && _partHeader.hasWeights)
+		if(_weightPtrIsOk)
 			_weightFile.seekg(_partHeader.channelCount * _polarizationCountInFile * sizeof(float), std::ios::cur);
 		_weightPtrIsOk = true;
 	}
@@ -201,15 +201,10 @@ void PartitionedMS::WriteModel(size_t rowId, std::complex<float>* buffer)
 	if(!_partHeader.hasModel)
 		throw std::runtime_error("Partitioned MS initialized without model");
 #endif
-	_weightFile.seekg(_partHeader.channelCount * _polarizationCountInFile * sizeof(float) * rowId, std::ios::beg);
-	_weightFile.read(reinterpret_cast<char*>(_weightBuffer.data()), _partHeader.channelCount * _polarizationCountInFile * sizeof(float));
-	for(size_t i=0; i!=_partHeader.channelCount * _polarizationCountInFile; ++i)
-		buffer[i] *= _weightBuffer[i];
-	
 	size_t rowLength = _partHeader.channelCount * _polarizationCountInFile * sizeof(std::complex<float>);
 	std::complex<float>* modelWritePtr = reinterpret_cast<std::complex<float>*>(_modelFileMap + rowLength*rowId);
 	
-	// In case the value was not sampled in this pass, it will be set to infinite and should not overwrite the current
+	// In case the value was not sampled in this pass, it has been set to infinite and should not overwrite the current
 	// value in the set.
 	for(size_t i=0; i!=_partHeader.channelCount * _polarizationCountInFile; ++i)
 	{
@@ -322,7 +317,7 @@ struct PartitionFiles
  * - Start channel in MS
  * - Total weight in part
  * - Data    (single polarization, as requested)
- * - Weights (single, only needed when imaging PSF)
+ * - Weights (single)
  * - Model, optionally
  */
 PartitionedMS::Handle PartitionedMS::Partition(const string& msPath, const std::vector<ChannelRange>& channels, MSSelection& selection, const string& dataColumnName, bool includeModel, bool initialModelRequired, const WSCleanSettings& settings)
@@ -457,17 +452,17 @@ PartitionedMS::Handle PartitionedMS::Partition(const string& msPath, const std::
 				for(PolarizationEnum p : polsOut)
 				{
 					PartitionFiles& f = files[fileIndex];
-					copyWeightedData(dataBuffer.data(), partStartCh, partEndCh, msPolarizations, dataArray, weightSpectrumArray, flagArray, p);
+					copyData(dataBuffer.data(), partStartCh, partEndCh, msPolarizations, dataArray, p);
 					f.data->write(reinterpret_cast<char*>(dataBuffer.data()), (partEndCh - partStartCh) * sizeof(std::complex<float>) * polarizationsPerFile);
 					if(!f.data->good())
 						throw std::runtime_error("Error writing to temporary data file");
 					
 					if(initialModelRequired)
 					{
-						copyWeightedData(dataBuffer.data(), partStartCh, partEndCh, msPolarizations, modelArray, weightSpectrumArray, flagArray, p);
+						copyData(dataBuffer.data(), partStartCh, partEndCh, msPolarizations, modelArray, p);
 						f.model->write(reinterpret_cast<char*>(dataBuffer.data()), (partEndCh - partStartCh) * sizeof(std::complex<float>) * polarizationsPerFile);
 						if(!f.model->good())
-							throw std::runtime_error("Error writing to temporary data file");
+							throw std::runtime_error("Error writing to temporary model data file");
 					}
 					
 					copyWeights(weightBuffer.data(), partStartCh, partEndCh, msPolarizations, dataArray, weightSpectrumArray, flagArray, p);
@@ -506,7 +501,6 @@ PartitionedMS::Handle PartitionedMS::Partition(const string& msPath, const std::
 	PartHeader header;
 	memset(&header, 0, sizeof(PartHeader));
 	header.hasModel = includeModel;
-	header.hasWeights = true;
 	fileIndex = 0;
 	dataBuffer.assign(channelCount * polarizationsPerFile, 0.0);
 	std::unique_ptr<ProgressBar> progress2;
@@ -580,7 +574,7 @@ void PartitionedMS::unpartition(const PartitionedMS::Handle::HandleData& handle)
 		const size_t channelParts = handle._channels.size();
 		
 		// Open the temporary files
-		std::vector<std::ifstream*> modelFiles(channelParts*pols.size()), weightFiles(channelParts*pols.size());
+		std::vector<std::unique_ptr<std::ifstream>> modelFiles(channelParts*pols.size());
 		size_t fileIndex = 0;
 		for(size_t part=0; part!=channelParts; ++part)
 		{
@@ -588,9 +582,7 @@ void PartitionedMS::unpartition(const PartitionedMS::Handle::HandleData& handle)
 			for(std::set<PolarizationEnum>::const_iterator p=pols.begin(); p!=pols.end(); ++p)
 			{
 				std::string partPrefix = getPartPrefix(handle._msPath, part, *p, dataDescId, handle._temporaryDirectory);
-				modelFiles[fileIndex] = new std::ifstream(partPrefix + "-m.tmp");
-				if(firstPartHeader.hasWeights)
-					weightFiles[fileIndex] = new std::ifstream(partPrefix + "-w.tmp");
+				modelFiles[fileIndex].reset(new std::ifstream(partPrefix + "-m.tmp"));
 				++fileIndex;
 			}
 		}
@@ -623,7 +615,7 @@ void PartitionedMS::unpartition(const PartitionedMS::Handle::HandleData& handle)
 		size_t selectedRowCountForDebug = 0;
 		for(size_t row=startRow; row!=endRow; ++row)
 		{
-			progress.SetProgress(row - startRow, startRow - endRow);
+			progress.SetProgress(row - startRow, endRow - startRow);
 			const int
 				a1 = antenna1Column(row), a2 = antenna2Column(row),
 				fieldId = fieldIdColumn(row), dataDescId = dataDescIdColumn(row);
@@ -654,14 +646,6 @@ void PartitionedMS::unpartition(const PartitionedMS::Handle::HandleData& handle)
 								modelFiles[fileIndex]->read(reinterpret_cast<char*>(modelDataBuffer.data()), (partEndCh - partStartCh) * polarizationsPerFile * sizeof(std::complex<float>));
 								if(!modelFiles[fileIndex]->good())
 									throw std::runtime_error("Error reading from temporary model data file");
-								if(firstPartHeader.hasWeights)
-								{
-									weightFiles[fileIndex]->read(reinterpret_cast<char*>(weightBuffer.data()), (partEndCh - partStartCh) * polarizationsPerFile * sizeof(float));
-									if(!weightFiles[fileIndex]->good())
-										throw std::runtime_error("Error reading from temporary weight data file");
-									for(size_t i=0; i!=(partEndCh - partStartCh) * polarizationsPerFile; ++i)
-										modelDataBuffer[i] /= weightBuffer[i];
-								}
 								reverseCopyData(modelDataArray, partStartCh, partEndCh, msPolarizations, modelDataBuffer.data(), *p);
 								
 								++fileIndex;
@@ -679,24 +663,12 @@ void PartitionedMS::unpartition(const PartitionedMS::Handle::HandleData& handle)
 		progress.SetProgress(ms.nrow(),ms.nrow());
 		
 		Logger::Debug << "Row count during unpartitioning: " << selectedRowCountForDebug << '\n';
-		
-		fileIndex = 0;
-		for(size_t part=0; part!=channelParts; ++part)
-		{
-			for(std::set<PolarizationEnum>::const_iterator p=pols.begin(); p!=pols.end(); ++p)
-			{
-				delete modelFiles[fileIndex];
-				if(firstPartHeader.hasWeights)
-					delete weightFiles[fileIndex];
-				++fileIndex;
-			}
-		}
 	}
 }
 
 PartitionedMS::Handle::HandleData::~HandleData()
 {
-	if(_modelUpdateRequired && !_initialModelRequired)
+	if(_modelUpdateRequired)
 		PartitionedMS::unpartition(*this);
 	
 	Logger::Info << "Cleaning up temporary files...\n";
