@@ -74,13 +74,13 @@ BOOST_AUTO_TEST_CASE( entriesInGroup )
 
 BOOST_AUTO_TEST_CASE( psfCount1 )
 {
-	ImageSet dset(&table, allocator, 1, false, 2, 2);
+	ImageSet dset(&table, allocator, 1, false, std::set<PolarizationEnum>(), 2, 2);
 	BOOST_CHECK_EQUAL(dset.PSFCount(), 1);
 }
 
 BOOST_AUTO_TEST_CASE( psfCount2 )
 {
-	ImageSet dset(&table, allocator, 2, false, 2, 2);
+	ImageSet dset(&table, allocator, 2, false, std::set<PolarizationEnum>(), 2, 2);
 	BOOST_CHECK_EQUAL(dset.PSFCount(), 2);
 }
 
@@ -114,7 +114,7 @@ BOOST_FIXTURE_TEST_CASE( load , AdvImageSetFixture)
 	
 BOOST_FIXTURE_TEST_CASE( loadAndAverage , AdvImageSetFixture)
 {
-	ImageSet dset(&table, allocator, 1, false, 2, 2);
+	ImageSet dset(&table, allocator, 1, false, std::set<PolarizationEnum>(), 2, 2);
 	dset.LoadAndAverage(cSet);
 	BOOST_CHECK_CLOSE_FRACTION(dset[0][0], 0.5*( 2.0 + 20.0), 1e-8);
 	BOOST_CHECK_CLOSE_FRACTION(dset[1][0], 0.5*(-1.0 - 10.0), 1e-8);
@@ -122,7 +122,7 @@ BOOST_FIXTURE_TEST_CASE( loadAndAverage , AdvImageSetFixture)
 
 BOOST_FIXTURE_TEST_CASE( interpolateAndStore , AdvImageSetFixture)
 {
-	ImageSet dset(&table, allocator, 2, false, 2, 2);
+	ImageSet dset(&table, allocator, 2, false, std::set<PolarizationEnum>(), 2, 2);
 	SpectralFitter fitter(NoSpectralFitting, 2);
 	dset.LoadAndAverage(cSet);
 	dset.InterpolateAndStore(cSet, fitter);
@@ -134,7 +134,8 @@ BOOST_FIXTURE_TEST_CASE( xxNormalization , ImageSetFixtureBase )
 {
 	addToImageSet(table, 0, 0, 0, 0, Polarization::XX, 100);
 	table.Update();
-	ImageSet dset(&table, allocator, 1, false, 2, 2);
+	std::set<PolarizationEnum> pols{ Polarization::XX };
+	ImageSet dset(&table, allocator, 1, false, pols, 2, 2);
 	dset[0][1] = 5.0;
 	checkLinearValue(1, 5.0, dset);
 	checkSquaredValue(1, 5.0, dset);
@@ -144,7 +145,8 @@ BOOST_FIXTURE_TEST_CASE( iNormalization , ImageSetFixtureBase )
 {
 	addToImageSet(table, 0, 0, 0, 0, Polarization::StokesI, 100);
 	table.Update();
-	ImageSet dset(&table, allocator, 1, false, 2, 2);
+	std::set<PolarizationEnum> pols{ Polarization::StokesI };
+	ImageSet dset(&table, allocator, 1, false, pols, 2, 2);
 	dset[0][2] = 6.0;
 	checkLinearValue(2, 6.0, dset);
 	checkSquaredValue(2, 6.0, dset);
@@ -155,7 +157,8 @@ BOOST_FIXTURE_TEST_CASE( i_2channel_Normalization , ImageSetFixtureBase )
 	addToImageSet(table, 0, 0, 0, 0, Polarization::StokesI, 100);
 	addToImageSet(table, 1, 0, 1, 1, Polarization::StokesI, 200);
 	table.Update();
-	ImageSet dset(&table, allocator, 2, false, 2, 2);
+	std::set<PolarizationEnum> pols{ Polarization::StokesI };
+	ImageSet dset(&table, allocator, 2, false, pols, 2, 2);
 	dset[0][0] = 12.0;
 	dset[1][0] = 13.0;
 	checkLinearValue(0, 12.5, dset);
@@ -167,7 +170,8 @@ BOOST_FIXTURE_TEST_CASE( xxyyNormalization , ImageSetFixtureBase )
 	addToImageSet(table, 0, 0, 0, 0, Polarization::XX, 100);
 	addToImageSet(table, 1, 0, 0, 0, Polarization::YY, 100);
 	table.Update();
-	ImageSet dset(&table, allocator, 1, false, 2, 2);
+	std::set<PolarizationEnum> pols{ Polarization::XX, Polarization::YY };
+	ImageSet dset(&table, allocator, 1, false, pols, 2, 2);
 	dset[0][3] = 7.0;
 	dset[1][3] = 8.0;
 	checkLinearValue(3, 7.5, dset);
@@ -180,13 +184,27 @@ BOOST_FIXTURE_TEST_CASE( iqNormalization , ImageSetFixtureBase )
 	addToImageSet(table, 0, 0, 0, 0, Polarization::StokesI, 100);
 	addToImageSet(table, 1, 0, 0, 0, Polarization::StokesQ, 100);
 	table.Update();
-	ImageSet dset(&table, allocator, 1, false, 2, 2);
+	std::set<PolarizationEnum> pols{ Polarization::StokesI, Polarization::StokesQ };
+	ImageSet dset(&table, allocator, 1, false, pols, 2, 2);
 	dset[0][0] = 6.0;
 	dset[1][0] = -1.0;
 	checkLinearValue(0, 5.0, dset);
 	checkSquaredValue(0, sqrt(6.0*6.0 + -1.0*-1.0), dset);
 }
 
+BOOST_FIXTURE_TEST_CASE( linkedINormalization , ImageSetFixtureBase )
+{
+	addToImageSet(table, 0, 0, 0, 0, Polarization::StokesI, 100);
+	addToImageSet(table, 1, 0, 0, 0, Polarization::StokesQ, 100);
+	table.Update();
+	std::set<PolarizationEnum> pols{ Polarization::StokesI };
+	ImageSet dset(&table, allocator, 1, false, pols, 2, 2);
+	dset[0][0] = 3.0;
+	dset[1][0] = -1.0;
+	checkLinearValue(0, 3.0, dset);
+	checkSquaredValue(0, 3.0, dset);
+}
+
 BOOST_FIXTURE_TEST_CASE( iquvNormalization , ImageSetFixtureBase )
 {
 	addToImageSet(table, 0, 0, 0, 0, Polarization::StokesI, 100);
@@ -194,7 +212,11 @@ BOOST_FIXTURE_TEST_CASE( iquvNormalization , ImageSetFixtureBase )
 	addToImageSet(table, 2, 0, 0, 0, Polarization::StokesU, 100);
 	addToImageSet(table, 3, 0, 0, 0, Polarization::StokesV, 100);
 	table.Update();
-	ImageSet dset(&table, allocator, 1, false, 2, 2);
+	std::set<PolarizationEnum> pols{
+		Polarization::StokesI, Polarization::StokesQ,
+		Polarization::StokesU, Polarization::StokesV
+	};
+	ImageSet dset(&table, allocator, 1, false, pols, 2, 2);
 	dset[0][0] = 9.0;
 	dset[1][0] = 0.2;
 	dset[2][0] = 0.2;
@@ -210,7 +232,11 @@ BOOST_FIXTURE_TEST_CASE( xx_xy_yx_yyNormalization , ImageSetFixtureBase )
 	addToImageSet(table, 2, 0, 0, 0, Polarization::YX, 100);
 	addToImageSet(table, 3, 0, 0, 0, Polarization::YY, 100);
 	table.Update();
-	ImageSet dset(&table, allocator, 1, false, 2, 2);
+	std::set<PolarizationEnum> pols{
+		Polarization::XX, Polarization::XY,
+		Polarization::YX, Polarization::YY
+	};
+	ImageSet dset(&table, allocator, 1, false, pols, 2, 2);
 	dset[0][1] = 10.0;
 	dset[1][1] = 0.25;
 	dset[2][1] = 0.25;
@@ -230,7 +256,11 @@ BOOST_FIXTURE_TEST_CASE( xx_xy_yx_yy_2channel_Normalization , ImageSetFixtureBas
 	addToImageSet(table, 6, 0, 1, 1, Polarization::YX, 200);
 	addToImageSet(table, 7, 0, 1, 1, Polarization::YY, 200);
 	table.Update();
-	ImageSet dset(&table, allocator, 2, false, 2, 2);
+	std::set<PolarizationEnum> pols{
+		Polarization::XX, Polarization::XY,
+		Polarization::YX, Polarization::YY
+	};
+	ImageSet dset(&table, allocator, 2, false, pols, 2, 2);
 	dset[0][2] = 5.0;
 	dset[1][2] = 0.1;
 	dset[2][2] = 0.2;
@@ -251,4 +281,62 @@ BOOST_FIXTURE_TEST_CASE( xx_xy_yx_yy_2channel_Normalization , ImageSetFixtureBas
 	checkSquaredValue(2, (sqrt(sqVal1*0.5) + sqrt(sqVal2*0.5))*0.5, dset);
 }
 
+BOOST_FIXTURE_TEST_CASE( linked_xx_yy_2channel_Normalization , ImageSetFixtureBase )
+{
+	addToImageSet(table, 0, 0, 0, 0, Polarization::XX, 100);
+	addToImageSet(table, 1, 0, 0, 0, Polarization::XY, 100);
+	addToImageSet(table, 2, 0, 0, 0, Polarization::YX, 100);
+	addToImageSet(table, 3, 0, 0, 0, Polarization::YY, 100);
+	addToImageSet(table, 4, 0, 1, 1, Polarization::XX, 200);
+	addToImageSet(table, 5, 0, 1, 1, Polarization::XY, 200);
+	addToImageSet(table, 6, 0, 1, 1, Polarization::YX, 200);
+	addToImageSet(table, 7, 0, 1, 1, Polarization::YY, 200);
+	table.Update();
+	std::set<PolarizationEnum> pols{
+		Polarization::XX, Polarization::YY
+	};
+	ImageSet dset(&table, allocator, 2, false, pols, 2, 2);
+	dset[0][2] = 7.5;
+	dset[1][2] = 0.1;
+	dset[2][2] = -0.2;
+	dset[3][2] = 6.5;
+	dset[4][2] = 8.5;
+	dset[5][2] = 0.3;
+	dset[6][2] = -0.4;
+	dset[7][2] = 9.5;
+	double
+		sqVal1 = dset[0][2] * dset[0][2] + dset[3][2] * dset[3][2],
+		sqVal2 = dset[4][2] * dset[4][2] + dset[7][2] * dset[7][2];
+	checkLinearValue(2, 32.0*0.25, dset);
+	checkSquaredValue(2, (sqrt(sqVal1*0.5) + sqrt(sqVal2*0.5))*0.5, dset);
+}
+
+BOOST_FIXTURE_TEST_CASE( linked_xx_2channel_Normalization , ImageSetFixtureBase )
+{
+	addToImageSet(table, 0, 0, 0, 0, Polarization::XX, 100);
+	addToImageSet(table, 1, 0, 0, 0, Polarization::XY, 100);
+	addToImageSet(table, 2, 0, 0, 0, Polarization::YX, 100);
+	addToImageSet(table, 3, 0, 0, 0, Polarization::YY, 100);
+	addToImageSet(table, 4, 0, 1, 1, Polarization::XX, 200);
+	addToImageSet(table, 5, 0, 1, 1, Polarization::XY, 200);
+	addToImageSet(table, 6, 0, 1, 1, Polarization::YX, 200);
+	addToImageSet(table, 7, 0, 1, 1, Polarization::YY, 200);
+	table.Update();
+	std::set<PolarizationEnum> pols{ Polarization::XX };
+	ImageSet dset(&table, allocator, 2, false, pols, 2, 2);
+	dset[0][2] = 7.5;
+	dset[1][2] = 0.1;
+	dset[2][2] = -0.2;
+	dset[3][2] = 6.5;
+	dset[4][2] = 8.5;
+	dset[5][2] = 0.3;
+	dset[6][2] = -0.4;
+	dset[7][2] = 9.5;
+	double
+		sqVal1 = dset[0][2] * dset[0][2],
+		sqVal2 = dset[4][2] * dset[4][2];
+	checkLinearValue(2, 32.0*0.25, dset);
+	checkSquaredValue(2, (sqrt(sqVal1) + sqrt(sqVal2))*0.5, dset);
+}
+
 BOOST_AUTO_TEST_SUITE_END()
