@@ -7,13 +7,14 @@ struct OutputChannelInfo {
 		normalizationFactor(1.0),
 		wGridSize(0), visibilityCount(0),
 		effectiveVisibilityCount(0.0),
+		visibilityWeightSum(0.0),
 		beamMaj(0.0), beamMin(0.0), beamPA(0.0),
 		theoreticBeamSize(0.0),
 		psfNormalizationFactor(1.0)
 	{ }
 	double weight, normalizationFactor;
 	size_t wGridSize, visibilityCount;
-	double effectiveVisibilityCount;
+	double effectiveVisibilityCount, visibilityWeightSum;
 	double beamMaj, beamMin, beamPA;
 	double theoreticBeamSize, psfNormalizationFactor;
 };
